"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createArrayComponent = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var destroyer_1 = require("../../helpers/destroyer");
var ReplayDynamicEntry_1 = require("./dynamic-entry/ReplayDynamicEntry");
var helpers_1 = require("./helpers");
function createArrayComponent() {
    var update$ = new rxjs_1.ReplaySubject(1);
    var _a = tslib_1.__read(destroyer_1.createDestroyer(), 2), destroy = _a[0], destroy$ = _a[1];
    var items$ = new rxjs_1.Observable(function (observer) {
        var dynamicEntries = new Map();
        destroy$.subscribe(function () {
            var e_1, _a;
            try {
                for (var _b = tslib_1.__values(dynamicEntries.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var value = _c.value;
                    value.destroy();
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
        return update$
            .pipe(operators_1.startWith(null), operators_1.pairwise(), operators_1.switchMap(function (_a) {
            // NOTE: this code block is similar to Array Rendering logic
            // TODO: refactor
            var e_2, _b;
            var _c = tslib_1.__read(_a, 2), prev = _c[0], curr = _c[1];
            // shortcut
            // if curr array is empty -- just return empty array
            if (curr.length == 0) {
                try {
                    for (var _d = tslib_1.__values(dynamicEntries.values()), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var entry = _e.value;
                        entry.destroy();
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_e && !_e.done && (_b = _d.return)) _b.call(_d);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
                dynamicEntries.clear();
                return rxjs_1.of([]);
            }
            // shortcut
            // if all elements (keys) are the same -- just push an update to them
            // NOTE: prev.every might be costly
            if (prev &&
                prev.length == curr.length &&
                prev.every(function (p, i) {
                    return Object.is(p.props.key, curr[i].props.key);
                })) {
                curr.forEach(function (definition) {
                    var key = definition.props.key;
                    dynamicEntries.get(key).update$.next(definition);
                });
                return rxjs_1.EMPTY;
            }
            // removing obsolete keys
            if (prev && prev.length != 0) {
                for (var prevIndex = 0; prevIndex < prev.length; prevIndex++) {
                    var shouldRemove = true;
                    var prevKey = prev[prevIndex].props.key;
                    for (var currKey = 0; currKey < curr.length; currKey++) {
                        if (Object.is(prevKey, curr[currKey].props.key)) {
                            shouldRemove = false;
                            break;
                        }
                    }
                    if (shouldRemove) {
                        var dynamicEntry = dynamicEntries.get(prevKey);
                        dynamicEntry.destroy();
                        dynamicEntries.delete(prevKey);
                    }
                }
            }
            var observableItems = curr.map(function (definition) {
                var key = definition.props.key;
                if (key == null ||
                    typeof key == 'object' ||
                    typeof key == 'function') {
                    var err = new Error('Key should be string | number | bigint | boolean | Symbol');
                    // TODO: error only in dev mode
                    console.error(err);
                    throw err;
                }
                var dynamicEntry = dynamicEntries.get(key);
                if (dynamicEntry == undefined) {
                    dynamicEntry = ReplayDynamicEntry_1.ReplayDynamicEntry();
                    dynamicEntries.set(key, dynamicEntry);
                }
                return new rxjs_1.Observable(function (observer) {
                    var subscription = dynamicEntry.result$
                        .pipe(operators_1.distinctUntilChanged(), operators_1.map(function (component) { return ({ key: key, component: component }); }))
                        .subscribe(observer);
                    dynamicEntry.connect();
                    // this is pushed before arr child is rendered
                    dynamicEntry.update$.next(definition);
                    return subscription;
                });
            });
            return rxjs_1.combineLatest.apply(void 0, tslib_1.__spread(observableItems));
        }), operators_1.takeUntil(destroy$))
            .subscribe(observer);
    });
    return {
        type: helpers_1.ComponentType.array,
        // lifecycle
        update$: update$,
        destroy: destroy,
        // out
        items$: items$,
    };
}
exports.createArrayComponent = createArrayComponent;
