"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFnComponent = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var asObservable_1 = require("../../helpers/asObservable");
var destroyer_1 = require("../../helpers/destroyer");
var DynamicEntry_1 = require("./dynamic-entry/DynamicEntry");
var helpers_1 = require("./helpers");
function createFnComponent(definition) {
    var update$ = new rxjs_1.ReplaySubject(1);
    var _a = tslib_1.__read(destroyer_1.createDestroyer(), 2), destroy = _a[0], destroy$ = _a[1];
    var props$ = update$.pipe(operators_1.pluck('props'), operators_1.takeUntil(destroy$));
    var result$ = new rxjs_1.Observable(function (observer) {
        // WARNING: Prop threads is an experimental feature
        // We keep created observables so that props.stream === props.stream
        var propThreadMap = new Map();
        var proxiedProps = new Proxy(props$, {
            get: function (target, prop, receiver) {
                // props$ Observable API
                if (Reflect.has(target, prop)) {
                    return Reflect.get(target, prop, receiver);
                }
                // Prop thread
                var observable = propThreadMap.get(prop);
                if (observable !== undefined) {
                    return observable;
                }
                observable = target.pipe(operators_1.map(function (o) { return o && Reflect.get(o, prop); }), operators_1.distinctUntilChanged(), operators_1.switchMap(asObservable_1.asObservable), operators_1.takeUntil(destroy$));
                propThreadMap.set(prop, observable);
                return observable;
            },
        });
        var dynamicRoot = DynamicEntry_1.DynamicEntry();
        dynamicRoot.result$.subscribe(observer);
        destroy$.subscribe(dynamicRoot.destroy);
        // run the component fn
        var result = definition.type(proxiedProps, { destroy$: destroy$ });
        // TODO: check if the value can be rendered
        // (is an Element or a basic type)
        // and throw otherwise
        return asObservable_1.asObservable(result)
            .pipe(operators_1.takeUntil(destroy$))
            .subscribe(dynamicRoot.update$);
    });
    return {
        type: helpers_1.ComponentType.fn,
        // lifecycle
        update$: update$,
        destroy: destroy,
        // out
        result$: result$,
    };
}
exports.createFnComponent = createFnComponent;
