"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicEntry = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var __1 = require("..");
var destroyer_1 = require("../../../helpers/destroyer");
var Element_1 = require("../../Element");
var helpers_1 = require("../helpers");
var NULL_CHILD_STUB = Object.create(null);
exports.DynamicEntry = function () {
    // NOTE: on using ReplaySubject instead of Subject here and in components:
    // when rendering an array of elements we use combineLatest, which waits
    // till all elements components to emit. Therefore rendering of subelements
    // would be delayed, which means that we subscribe to sub-components after
    // we subscribe to higher dynamic elements (later triger pushing updates to
    // to former).
    // This brings some issues:
    // If we use ReplaySubject(1) -- then we might miss some updates. If client
    // code in Fn will subscribe to props$ w/ a delay -- they will receive all
    // previous updates.
    var update$ = new rxjs_1.ReplaySubject(1);
    var _a = tslib_1.__read(destroyer_1.createDestroyer(), 2), destroy = _a[0], destroy$ = _a[1];
    var component;
    var prev = NULL_CHILD_STUB;
    destroy$.subscribe(function () {
        if (component) {
            component.destroy();
        }
    });
    var result$ = new rxjs_1.Observable(function (observer) {
        return update$.pipe(operators_1.takeUntil(destroy$)).subscribe({
            next: function (curr) {
                // create a new component if:
                // - there were no prev value
                // - different data types of children
                // - different types of elements (leaf / array / static / observable / fn)
                // - keys mismatch
                // - xmlns mismatch
                // - number of children mismatch
                if (prev === NULL_CHILD_STUB ||
                    helpers_1.getType(prev) != helpers_1.getType(curr) ||
                    (Element_1.isElement(curr) &&
                        Element_1.isElement(prev) &&
                        (prev.type !== curr.type ||
                            !Object.is(prev.props.key, curr.props.key) ||
                            !Object.is(prev.props.xmlns, curr.props.xmlns) ||
                            prev.props.children.length !==
                                curr.props.children.length))) {
                    if (component) {
                        component.destroy();
                    }
                    component = __1.createComponent(curr);
                    observer.next(component);
                }
                component.update$.next(curr);
                prev = curr;
            },
            complete: function () {
                observer.complete();
            },
            error: function (err) {
                observer.error(err);
            },
        });
    });
    return {
        update$: update$,
        result$: result$,
        destroy: destroy,
    };
};
