"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderStatic = exports.isHTMLRenderElement = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("../../constants");
var DomElement_1 = require("../../dom/DomElement");
var UpdateDomChildNodesPipe_1 = require("../../dom/UpdateDomChildNodesPipe");
var splitPropsToStreams_1 = require("../../helpers/splitPropsToStreams");
var index_1 = require("./index");
exports.isHTMLRenderElement = function (element) {
    return element && 'type' in element && element.type == 'Element';
};
// TODO: ensure all subscriptions are destroyed with returning observable
// watch updates
//    update domElement
// subscribe to child updates
//    for each child udpate -- render component( target = domElement )
function renderStatic(component, xmlns) {
    // xmlns is always the same for the same component
    // due to differentiation in DynamicEntry
    xmlns = component.definition.props.xmlns || xmlns;
    if (component.definition.type === 'svg' &&
        !component.definition.props.xmlns) {
        xmlns = 'http://www.w3.org/2000/svg';
    }
    var htmlElement = DomElement_1.createDomElement(component.definition.type, xmlns);
    var childrenXmlns = component.definition.type === 'foreignObject' ? null : xmlns;
    return new rxjs_1.Observable(function (observer) {
        // NOTE: perf optimisation:
        // to make first render faster, we wait for all children to emit their
        // first value (vDOM) and only then we append all emissions to the parent
        rxjs_1.combineLatest.apply(void 0, tslib_1.__spread(component.dynamicChildren.map(function (dynamicChild) {
            return dynamicChild.result$.pipe(operators_1.switchMap(function (result) {
                return index_1.renderComponent(result, childrenXmlns);
            }));
        }))).pipe(UpdateDomChildNodesPipe_1.updateDomChildNodesPipe(htmlElement))
            .subscribe();
        // Updating dom element
        component.change$
            .pipe(splitPropsToStreams_1.splitPropsToStreams(), 
        // subscribe to each stream and update DOM accordingly
        operators_1.flatMap(function (group) {
            var key = group.key;
            if (constants_1.PRESERVED_KEYS.includes(group.key)) {
                return rxjs_1.EMPTY;
            }
            var distinct$ = group.pipe(operators_1.distinctUntilChanged(), operators_1.startWith(void 0), operators_1.share());
            var isEvent = DomElement_1.isEventAttr(group.key);
            if (!isEvent) {
                return distinct$.pipe(operators_1.tap({
                    next: function (value) {
                        DomElement_1.updateAttribute(htmlElement, key, value);
                    },
                    complete: function () {
                        DomElement_1.removeAttribute(htmlElement, key);
                    },
                }));
            }
            else {
                var eventName_1 = isEvent && DomElement_1.getEventFromAttr(key);
                var latestValue_1;
                return distinct$.pipe(operators_1.pairwise(), operators_1.tap({
                    next: function (_a) {
                        var _b = tslib_1.__read(_a, 2), a = _b[0], b = _b[1];
                        latestValue_1 = b;
                        DomElement_1.updateEventListener(htmlElement, eventName_1, a, b);
                    },
                    complete: function () {
                        DomElement_1.removeEventListener(htmlElement, eventName_1, latestValue_1);
                    },
                }));
            }
        }))
            .subscribe();
        observer.next({
            type: 'Element',
            element$: component.element$,
            htmlElement: htmlElement,
        });
        observer.complete();
    });
}
exports.renderStatic = renderStatic;
