"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.asObservable = void 0;
var rxjs_1 = require("rxjs");
var isThenable_1 = require("./isThenable");
/**
 * Will ensure the value is Observable or will turn it into an Observable otherwise
 * NOTE: Promises will be wrapped into an Observable that emits or errors it's result
 * @param value Observable, Promise or any
 */
function asObservable(value) {
    // an Observable
    if (rxjs_1.isObservable(value)) {
        return value;
    }
    // a Promise
    // basically RxJS way of checking against Promise
    // https://github.com/ReactiveX/rxjs/blob/8ad3267fb337a22c383a4ff8901e03b0ae3e2a7a/src/internal/util/isPromise.ts
    if (isThenable_1.isThenable(value)) {
        return rxjs_1.from(value);
    }
    // otherwise
    // box in an Observable
    return rxjs_1.of(value);
}
exports.asObservable = asObservable;
