import { combineLatest, Observable, ReplaySubject } from 'rxjs';
import { map, pluck, takeUntil } from 'rxjs/operators';
import { createDestroyer } from '../../helpers/destroyer';
import { DynamicEntry } from './dynamic-entry/DynamicEntry';
import { ComponentType } from './helpers';
export const createStaticComponent = (element) => {
    const update$ = new ReplaySubject(1);
    const [destroy, destroy$] = createDestroyer();
    const element$ = new ReplaySubject(1);
    const dynamicChildren = element.props.children.map(() => DynamicEntry());
    destroy$.subscribe(() => {
        dynamicChildren.forEach((child) => child.destroy());
        element$.complete();
    });
    const change$ = new Observable((observer) => {
        dynamicChildren.forEach((child, i) => {
            update$
                .pipe(map((update) => update.props.children[i]), takeUntil(destroy$))
                .subscribe(child.update$);
        });
        // Ref handling
        // TODO: consider making ref a function instead of Subject
        combineLatest(update$.pipe(pluck('props', 'ref')), element$)
            .pipe(takeUntil(destroy$))
            .subscribe(([ref, element]) => {
            if (!ref || typeof ref.next != 'function') {
                return;
            }
            ref.next(element);
        });
        return update$
            .pipe(pluck('props'), takeUntil(destroy$))
            .subscribe(observer);
    });
    return {
        type: ComponentType.static,
        definition: element,
        // lifecycle
        update$,
        element$,
        destroy,
        // output
        change$,
        dynamicChildren,
    };
};
