import { __read, __spread, __values } from "tslib";
import { combineLatest, EMPTY, Observable, of, ReplaySubject } from 'rxjs';
import { distinctUntilChanged, map, pairwise, startWith, switchMap, takeUntil, } from 'rxjs/operators';
import { createDestroyer } from '../../helpers/destroyer';
import { ReplayDynamicEntry, } from './dynamic-entry/ReplayDynamicEntry';
import { ComponentType } from './helpers';
export function createArrayComponent() {
    var update$ = new ReplaySubject(1);
    var _a = __read(createDestroyer(), 2), destroy = _a[0], destroy$ = _a[1];
    var items$ = new Observable(function (observer) {
        var dynamicEntries = new Map();
        destroy$.subscribe(function () {
            var e_1, _a;
            try {
                for (var _b = __values(dynamicEntries.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var value = _c.value;
                    value.destroy();
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
        return update$
            .pipe(startWith(null), pairwise(), switchMap(function (_a) {
            // NOTE: this code block is similar to Array Rendering logic
            // TODO: refactor
            var e_2, _b;
            var _c = __read(_a, 2), prev = _c[0], curr = _c[1];
            // shortcut
            // if curr array is empty -- just return empty array
            if (curr.length == 0) {
                try {
                    for (var _d = __values(dynamicEntries.values()), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var entry = _e.value;
                        entry.destroy();
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_e && !_e.done && (_b = _d.return)) _b.call(_d);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
                dynamicEntries.clear();
                return of([]);
            }
            // shortcut
            // if all elements (keys) are the same -- just push an update to them
            // NOTE: prev.every might be costly
            if (prev &&
                prev.length == curr.length &&
                prev.every(function (p, i) {
                    return Object.is(p.props.key, curr[i].props.key);
                })) {
                curr.forEach(function (definition) {
                    var key = definition.props.key;
                    dynamicEntries.get(key).update$.next(definition);
                });
                return EMPTY;
            }
            // removing obsolete keys
            if (prev && prev.length != 0) {
                for (var prevIndex = 0; prevIndex < prev.length; prevIndex++) {
                    var shouldRemove = true;
                    var prevKey = prev[prevIndex].props.key;
                    for (var currKey = 0; currKey < curr.length; currKey++) {
                        if (Object.is(prevKey, curr[currKey].props.key)) {
                            shouldRemove = false;
                            break;
                        }
                    }
                    if (shouldRemove) {
                        var dynamicEntry = dynamicEntries.get(prevKey);
                        dynamicEntry.destroy();
                        dynamicEntries.delete(prevKey);
                    }
                }
            }
            var observableItems = curr.map(function (definition) {
                var key = definition.props.key;
                if (key == null ||
                    typeof key == 'object' ||
                    typeof key == 'function') {
                    var err = new Error('Key should be string | number | bigint | boolean | Symbol');
                    // TODO: error only in dev mode
                    console.error(err);
                    throw err;
                }
                var dynamicEntry = dynamicEntries.get(key);
                if (dynamicEntry == undefined) {
                    dynamicEntry = ReplayDynamicEntry();
                    dynamicEntries.set(key, dynamicEntry);
                }
                return new Observable(function (observer) {
                    var subscription = dynamicEntry.result$
                        .pipe(distinctUntilChanged(), map(function (component) { return ({ key: key, component: component }); }))
                        .subscribe(observer);
                    dynamicEntry.connect();
                    // this is pushed before arr child is rendered
                    dynamicEntry.update$.next(definition);
                    return subscription;
                });
            });
            return combineLatest.apply(void 0, __spread(observableItems));
        }), takeUntil(destroy$))
            .subscribe(observer);
    });
    return {
        type: ComponentType.array,
        // lifecycle
        update$: update$,
        destroy: destroy,
        // out
        items$: items$,
    };
}
