import { __read } from "tslib";
import { Observable, ReplaySubject } from 'rxjs';
import { distinctUntilChanged, map, pluck, switchMap, takeUntil, } from 'rxjs/operators';
import { asObservable } from '../../helpers/asObservable';
import { createDestroyer } from '../../helpers/destroyer';
import { DynamicEntry } from './dynamic-entry/DynamicEntry';
import { ComponentType } from './helpers';
export function createFnComponent(definition) {
    var update$ = new ReplaySubject(1);
    var _a = __read(createDestroyer(), 2), destroy = _a[0], destroy$ = _a[1];
    var props$ = update$.pipe(pluck('props'), takeUntil(destroy$));
    var result$ = new Observable(function (observer) {
        // WARNING: Prop threads is an experimental feature
        // We keep created observables so that props.stream === props.stream
        var propThreadMap = new Map();
        var proxiedProps = new Proxy(props$, {
            get: function (target, prop, receiver) {
                // props$ Observable API
                if (Reflect.has(target, prop)) {
                    return Reflect.get(target, prop, receiver);
                }
                // Prop thread
                var observable = propThreadMap.get(prop);
                if (observable !== undefined) {
                    return observable;
                }
                observable = target.pipe(map(function (o) { return o && Reflect.get(o, prop); }), distinctUntilChanged(), switchMap(asObservable), takeUntil(destroy$));
                propThreadMap.set(prop, observable);
                return observable;
            },
        });
        var dynamicRoot = DynamicEntry();
        dynamicRoot.result$.subscribe(observer);
        destroy$.subscribe(dynamicRoot.destroy);
        // run the component fn
        var result = definition.type(proxiedProps, { destroy$: destroy$ });
        // TODO: check if the value can be rendered
        // (is an Element or a basic type)
        // and throw otherwise
        return asObservable(result)
            .pipe(takeUntil(destroy$))
            .subscribe(dynamicRoot.update$);
    });
    return {
        type: ComponentType.fn,
        // lifecycle
        update$: update$,
        destroy: destroy,
        // out
        result$: result$,
    };
}
