import { __read, __spread } from "tslib";
import { combineLatest, EMPTY, Observable, } from 'rxjs';
import { distinctUntilChanged, flatMap, pairwise, share, startWith, switchMap, tap, } from 'rxjs/operators';
import { PRESERVED_KEYS } from '../../constants';
import { createDomElement, getEventFromAttr, isEventAttr, removeAttribute, removeEventListener, updateAttribute, updateEventListener, } from '../../dom/DomElement';
import { updateDomChildNodesPipe } from '../../dom/UpdateDomChildNodesPipe';
import { splitPropsToStreams } from '../../helpers/splitPropsToStreams';
import { renderComponent } from './index';
export var isHTMLRenderElement = function (element) {
    return element && 'type' in element && element.type == 'Element';
};
// TODO: ensure all subscriptions are destroyed with returning observable
// watch updates
//    update domElement
// subscribe to child updates
//    for each child udpate -- render component( target = domElement )
export function renderStatic(component, xmlns) {
    // xmlns is always the same for the same component
    // due to differentiation in DynamicEntry
    xmlns = component.definition.props.xmlns || xmlns;
    if (component.definition.type === 'svg' &&
        !component.definition.props.xmlns) {
        xmlns = 'http://www.w3.org/2000/svg';
    }
    var htmlElement = createDomElement(component.definition.type, xmlns);
    var childrenXmlns = component.definition.type === 'foreignObject' ? null : xmlns;
    return new Observable(function (observer) {
        // NOTE: perf optimisation:
        // to make first render faster, we wait for all children to emit their
        // first value (vDOM) and only then we append all emissions to the parent
        combineLatest.apply(void 0, __spread(component.dynamicChildren.map(function (dynamicChild) {
            return dynamicChild.result$.pipe(switchMap(function (result) {
                return renderComponent(result, childrenXmlns);
            }));
        }))).pipe(updateDomChildNodesPipe(htmlElement))
            .subscribe();
        // Updating dom element
        component.change$
            .pipe(splitPropsToStreams(), 
        // subscribe to each stream and update DOM accordingly
        flatMap(function (group) {
            var key = group.key;
            if (PRESERVED_KEYS.includes(group.key)) {
                return EMPTY;
            }
            var distinct$ = group.pipe(distinctUntilChanged(), startWith(void 0), share());
            var isEvent = isEventAttr(group.key);
            if (!isEvent) {
                return distinct$.pipe(tap({
                    next: function (value) {
                        updateAttribute(htmlElement, key, value);
                    },
                    complete: function () {
                        removeAttribute(htmlElement, key);
                    },
                }));
            }
            else {
                var eventName_1 = isEvent && getEventFromAttr(key);
                var latestValue_1;
                return distinct$.pipe(pairwise(), tap({
                    next: function (_a) {
                        var _b = __read(_a, 2), a = _b[0], b = _b[1];
                        latestValue_1 = b;
                        updateEventListener(htmlElement, eventName_1, a, b);
                    },
                    complete: function () {
                        removeEventListener(htmlElement, eventName_1, latestValue_1);
                    },
                }));
            }
        }))
            .subscribe();
        observer.next({
            type: 'Element',
            element$: component.element$,
            htmlElement: htmlElement,
        });
        observer.complete();
    });
}
