import { Subscription } from 'rxjs';
import { map } from 'rxjs/operators';
import { updateDomChildNodesPipe } from '../../dom/UpdateDomChildNodesPipe';
import { ComponentType, createComponent } from '../component';
import { renderArray } from './Array';
import { renderFn } from './Fn';
import { renderLeaf } from './Leaf';
import { renderObservable } from './Observable';
import { renderStatic } from './Static';
// TODO: add memory leak test
//       unsubscription might not spread up the tree
export var render = function (definition, target) {
    var root = createComponent(definition);
    renderRootComponent(root, target).subscribe();
    root.update$.next(definition);
    var subscription = new Subscription();
    subscription.add(root.destroy);
    return subscription;
};
function renderRootComponent(component, target) {
    return renderComponent(component, null).pipe(map(function (el) { return [el]; }), updateDomChildNodesPipe(target));
}
export function renderComponent(component, xmlns) {
    if (component.type == ComponentType.leaf) {
        return renderLeaf(component);
    }
    else if (component.type == ComponentType.fn) {
        return renderFn(component, xmlns);
    }
    else if (component.type == ComponentType.static) {
        return renderStatic(component, xmlns);
    }
    else if (component.type == ComponentType.observable) {
        return renderObservable(component, xmlns);
    }
    else if (component.type == ComponentType.array) {
        return renderArray(component, xmlns);
    }
    throw 'Unknown component type';
}
